@@ -1,3 +1,9 @@
+__all__ = ["mode", "modes", "to_cumulative", "reversed_phi_ppf", "interval_phi", "to_phi", "to_microns", "cm",
+           "arithmetic", "geometric", "logarithmic", "logarithmic_fw57", "geometric_fw57", "scale_description",
+           "proportions_gsm", "proportions_ssc", "proportions_bgssc", "all_proportions",
+           "group_gsm_folk54", "group_ssc_folk54", "group_folk54", "GROUP_BP12_SYMBOL_MAP",
+           "group_gsm_bp12", "group_ssc_bp12", "group_bp12", "major_statistics", "all_statistics"]
+
 import string
 from typing import *
 
@@ -122,6 +128,21 @@ def to_microns(sizes_phi: ndarray):
     return np.exp2(-sizes_phi) * 1000.0
 
 
+def cm(classes_phi: ndarray, distribution: ndarray) -> Tuple[float, float]:
+    """
+    Get the C and M values of a grain size distribution.
+    C is the first percentile in phi scale, and M is the median in phi scale.
+
+    :param classes_phi: The grain size classes in phi values.
+    :param distribution: The frequency distribution of grain size classes.
+        Note, the sum of frequencies should be equal to 1.
+    :return: A tuple that contains the C and M values.
+    """
+    ppf = reversed_phi_ppf(classes_phi, distribution)
+    CM = ppf(0.99), ppf(0.5)
+    return CM
+
+
 # The following five formulas of calculating the statistical parameters referred to Blott & Pye (2001)'s work
 # DOI: 10.1002/esp.261
 def arithmetic(classes: ndarray, distribution: ndarray) -> Dict[str, float]:
@@ -154,6 +175,7 @@ def geometric(classes: ndarray, distribution: ndarray) -> Dict[str, Union[float,
         `dict(mean=..., std=..., skewness=..., kurtosis=..., std_description=..., skewness_description=...,
             kurtosis_description=...)`
     """
+
     def std_description(std_value):
         if std_value < 1.27:
             return "Very well sorted"
@@ -217,6 +239,7 @@ def logarithmic(classes_phi: ndarray, distribution: ndarray) -> Dict[str, Union[
         `dict(mean=..., std=..., skewness=..., kurtosis=..., std_description=..., skewness_description=...,
             kurtosis_description=...)`
     """
+
     def std_description(std_value):
         if std_value < 0.35:
             return "Very well sorted"
@@ -280,6 +303,7 @@ def logarithmic_fw57(_ppf: Callable[[Union[int, float, np.ndarray]], Union[int,
         `dict(mean=..., std=..., skewness=..., kurtosis=..., std_description=..., skewness_description=...,
             kurtosis_description=...)`
     """
+
     def ppf(x):
         if not isinstance(x, ndarray):
             x = np.array(x)
@@ -351,6 +375,7 @@ def geometric_fw57(_ppf: Callable[[Union[int, float, np.ndarray]], Union[int, fl
         `dict(mean=..., std=..., skewness=..., kurtosis=..., std_description=..., skewness_description=...,
             kurtosis_description=...)`
     """
+
     def log_ppf(x):
         if not isinstance(x, ndarray):
             x = np.array(x)
