@@ -6,8 +6,9 @@ from PySide6 import QtCore, QtGui, QtWidgets
 
 from ..statistics import all_statistics
 from ..models import Dataset, Sample
-from ..charts import (highlight_color, FrequencyChart, CumulativeChart, Frequency3DChart, DiagramChart,
-                      BP12SSCDiagramChart, BP12GSMDiagramChart, Folk54SSCDiagramChart, Folk54GSMDiagramChart)
+from ..charts import (highlight_color, FrequencyChart, Frequency3DChart, FrequencyHeatmap, CumulativeChart,
+                      DiagramChart, BP12SSCDiagramChart, BP12GSMDiagramChart, Folk54SSCDiagramChart,
+                      Folk54GSMDiagramChart, CMDiagramChart)
 
 
 class StatisticalAnalyzer(QtWidgets.QWidget):
@@ -60,108 +61,136 @@ class StatisticalAnalyzer(QtWidgets.QWidget):
         self.main_layout.setColumnStretch(1, 1)
         self.main_layout.setColumnStretch(2, 1)
         self.menu = QtWidgets.QMenu(self.data_table)
-        self.plot_cumulative_curve_menu = self.menu.addMenu(self.tr("Plot Cumulative Frequency Chart"))
-        self.cumulative_plot_selected_action = self.plot_cumulative_curve_menu.addAction(self.tr("Plot"))
+        self.plot_cumulative_menu = self.menu.addMenu(self.tr("Plot Cumulative Frequency Chart"))
+        self.cumulative_plot_selected_action = self.plot_cumulative_menu.addAction(self.tr("Plot"))
         self.cumulative_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.cumulative_curve_chart, self.selections, False))
-        self.cumulative_append_selected_action = self.plot_cumulative_curve_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.cumulative_chart, self.selections, False))
+        self.cumulative_append_selected_action = self.plot_cumulative_menu.addAction(self.tr("Append"))
         self.cumulative_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.cumulative_curve_chart, self.selections, True))
-        self.cumulative_plot_all_action = self.plot_cumulative_curve_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.cumulative_chart, self.selections, True))
+        self.cumulative_plot_all_action = self.plot_cumulative_menu.addAction(self.tr("Plot All"))
         self.cumulative_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.cumulative_curve_chart, self._dataset, False))
-        self.cumulative_append_all_action = self.plot_cumulative_curve_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.cumulative_chart, self._dataset, False))
+        self.cumulative_append_all_action = self.plot_cumulative_menu.addAction(self.tr("Append All"))
         self.cumulative_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.cumulative_curve_chart, self._dataset, True))
-        self.plot_frequency_curve_menu = self.menu.addMenu(self.tr("Plot Frequency Distribution Chart"))
-        self.frequency_plot_selected_action = self.plot_frequency_curve_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.cumulative_chart, self._dataset, True))
+        self.plot_frequency_menu = self.menu.addMenu(self.tr("Plot Frequency Distribution Chart"))
+        self.frequency_plot_selected_action = self.plot_frequency_menu.addAction(self.tr("Plot"))
         self.frequency_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_chart, self.selections, False))
-        self.frequency_append_selected_action = self.plot_frequency_curve_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.frequency_chart, self.selections, False))
+        self.frequency_append_selected_action = self.plot_frequency_menu.addAction(self.tr("Append"))
         self.frequency_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_chart, self.selections, True))
-        self.frequency_plot_all_action = self.plot_frequency_curve_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.frequency_chart, self.selections, True))
+        self.frequency_plot_all_action = self.plot_frequency_menu.addAction(self.tr("Plot All"))
         self.frequency_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_chart, self._dataset, False))
-        self.frequency_append_all_action = self.plot_frequency_curve_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.frequency_chart, self._dataset, False))
+        self.frequency_append_all_action = self.plot_frequency_menu.addAction(self.tr("Append All"))
         self.frequency_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_chart, self._dataset, True))
-        self.plot_frequency_curve_3D_menu = self.menu.addMenu(self.tr("Plot Frequency 3D Chart"))
-        self.frequency_3D_plot_selected_action = self.plot_frequency_curve_3D_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.frequency_chart, self._dataset, True))
+        self.plot_frequency_3D_menu = self.menu.addMenu(self.tr("Plot Frequency 3D Chart"))
+        self.frequency_3D_plot_selected_action = self.plot_frequency_3D_menu.addAction(self.tr("Plot"))
         self.frequency_3D_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_3D_chart, self.selections, False))
-        self.frequency_3D_append_selected_action = self.plot_frequency_curve_3D_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.frequency_3D_chart, self.selections, False))
+        self.frequency_3D_append_selected_action = self.plot_frequency_3D_menu.addAction(self.tr("Append"))
         self.frequency_3D_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_3D_chart, self.selections, True))
-        self.frequency_3D_plot_all_action = self.plot_frequency_curve_3D_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.frequency_3D_chart, self.selections, True))
+        self.frequency_3D_plot_all_action = self.plot_frequency_3D_menu.addAction(self.tr("Plot All"))
         self.frequency_3D_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_3D_chart, self._dataset, False))
-        self.frequency_3D_append_all_action = self.plot_frequency_curve_3D_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.frequency_3D_chart, self._dataset, False))
+        self.frequency_3D_append_all_action = self.plot_frequency_3D_menu.addAction(self.tr("Append All"))
         self.frequency_3D_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.frequency_curve_3D_chart, self._dataset, True))
-        self.folk54_GSM_diagram_menu = self.menu.addMenu(self.tr("Plot GSM Diagram (Folk, 1954)"))
-        self.folk54_GSM_plot_selected_action = self.folk54_GSM_diagram_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.frequency_3D_chart, self._dataset, True))
+        self.plot_frequency_heatmap_menu = self.menu.addMenu(self.tr("Plot Frequency Heatmap"))
+        self.frequency_heatmap_plot_selected_action = self.plot_frequency_heatmap_menu.addAction(self.tr("Plot"))
+        self.frequency_heatmap_plot_selected_action.triggered.connect(
+            lambda: self.plot_chart(self.frequency_heatmap, self.selections, False))
+        self.frequency_heatmap_append_selected_action = self.plot_frequency_heatmap_menu.addAction(self.tr("Append"))
+        self.frequency_heatmap_append_selected_action.triggered.connect(
+            lambda: self.plot_chart(self.frequency_heatmap, self.selections, True))
+        self.frequency_heatmap_plot_all_action = self.plot_frequency_heatmap_menu.addAction(self.tr("Plot All"))
+        self.frequency_heatmap_plot_all_action.triggered.connect(
+            lambda: self.plot_chart(self.frequency_heatmap, self._dataset, False))
+        self.frequency_heatmap_append_all_action = self.plot_frequency_heatmap_menu.addAction(self.tr("Append All"))
+        self.frequency_heatmap_append_all_action.triggered.connect(
+            lambda: self.plot_chart(self.frequency_heatmap, self._dataset, True))
+        self.folk54_GSM_menu = self.menu.addMenu(self.tr("Plot GSM Diagram (Folk, 1954)"))
+        self.folk54_GSM_plot_selected_action = self.folk54_GSM_menu.addAction(self.tr("Plot"))
         self.folk54_GSM_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_GSM_diagram_chart, self.selections, False))
-        self.folk54_GSM_append_selected_action = self.folk54_GSM_diagram_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.folk54_GSM_diagram, self.selections, False))
+        self.folk54_GSM_append_selected_action = self.folk54_GSM_menu.addAction(self.tr("Append"))
         self.folk54_GSM_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_GSM_diagram_chart, self.selections, True))
-        self.folk54_GSM_plot_all_action = self.folk54_GSM_diagram_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.folk54_GSM_diagram, self.selections, True))
+        self.folk54_GSM_plot_all_action = self.folk54_GSM_menu.addAction(self.tr("Plot All"))
         self.folk54_GSM_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_GSM_diagram_chart, self._dataset, False))
-        self.folk54_GSM_append_all_action = self.folk54_GSM_diagram_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.folk54_GSM_diagram, self._dataset, False))
+        self.folk54_GSM_append_all_action = self.folk54_GSM_menu.addAction(self.tr("Append All"))
         self.folk54_GSM_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_GSM_diagram_chart, self._dataset, True))
-        self.folk54_SSC_diagram_menu = self.menu.addMenu(self.tr("Plot SSC Diagram (Folk, 1954)"))
-        self.folk54_SSC_plot_selected_action = self.folk54_SSC_diagram_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.folk54_GSM_diagram, self._dataset, True))
+        self.folk54_SSC_menu = self.menu.addMenu(self.tr("Plot SSC Diagram (Folk, 1954)"))
+        self.folk54_SSC_plot_selected_action = self.folk54_SSC_menu.addAction(self.tr("Plot"))
         self.folk54_SSC_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_SSC_diagram_chart, self.selections, False))
-        self.folk54_SSC_append_selected_action = self.folk54_SSC_diagram_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.folk54_SSC_diagram, self.selections, False))
+        self.folk54_SSC_append_selected_action = self.folk54_SSC_menu.addAction(self.tr("Append"))
         self.folk54_SSC_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_SSC_diagram_chart, self.selections, True))
-        self.folk54_SSC_plot_all_action = self.folk54_SSC_diagram_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.folk54_SSC_diagram, self.selections, True))
+        self.folk54_SSC_plot_all_action = self.folk54_SSC_menu.addAction(self.tr("Plot All"))
         self.folk54_SSC_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_SSC_diagram_chart, self._dataset, False))
-        self.folk54_SSC_append_all_action = self.folk54_SSC_diagram_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.folk54_SSC_diagram, self._dataset, False))
+        self.folk54_SSC_append_all_action = self.folk54_SSC_menu.addAction(self.tr("Append All"))
         self.folk54_SSC_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.folk54_SSC_diagram_chart, self._dataset, True))
-        self.BP12_GSM_diagram_menu = self.menu.addMenu(self.tr("Plot GSM Diagram (Blott and Pye, 2012)"))
-        self.BP12_GSM_plot_selected_action = self.BP12_GSM_diagram_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.folk54_SSC_diagram, self._dataset, True))
+        self.BP12_GSM_menu = self.menu.addMenu(self.tr("Plot GSM Diagram (Blott and Pye, 2012)"))
+        self.BP12_GSM_plot_selected_action = self.BP12_GSM_menu.addAction(self.tr("Plot"))
         self.BP12_GSM_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_GSM_diagram_chart, self.selections, False))
-        self.BP12_GSM_append_selected_action = self.BP12_GSM_diagram_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.BP12_GSM_diagram, self.selections, False))
+        self.BP12_GSM_append_selected_action = self.BP12_GSM_menu.addAction(self.tr("Append"))
         self.BP12_GSM_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_GSM_diagram_chart, self.selections, True))
-        self.BP12_GSM_plot_all_action = self.BP12_GSM_diagram_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.BP12_GSM_diagram, self.selections, True))
+        self.BP12_GSM_plot_all_action = self.BP12_GSM_menu.addAction(self.tr("Plot All"))
         self.BP12_GSM_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_GSM_diagram_chart, self._dataset, False))
-        self.BP12_GSM_append_all_action = self.BP12_GSM_diagram_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.BP12_GSM_diagram, self._dataset, False))
+        self.BP12_GSM_append_all_action = self.BP12_GSM_menu.addAction(self.tr("Append All"))
         self.BP12_GSM_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_GSM_diagram_chart, self._dataset, True))
-        self.BP12_SSC_diagram_menu = self.menu.addMenu(self.tr("Plot SSC Diagram (Blott and Pye, 2012)"))
-        self.BP12_SSC_plot_selected_action = self.BP12_SSC_diagram_menu.addAction(self.tr("Plot"))
+            lambda: self.plot_chart(self.BP12_GSM_diagram, self._dataset, True))
+        self.BP12_SSC_menu = self.menu.addMenu(self.tr("Plot SSC Diagram (Blott and Pye, 2012)"))
+        self.BP12_SSC_plot_selected_action = self.BP12_SSC_menu.addAction(self.tr("Plot"))
         self.BP12_SSC_plot_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_SSC_diagram_chart, self.selections, False))
-        self.BP12_SSC_append_selected_action = self.BP12_SSC_diagram_menu.addAction(self.tr("Append"))
+            lambda: self.plot_chart(self.BP12_SSC_diagram, self.selections, False))
+        self.BP12_SSC_append_selected_action = self.BP12_SSC_menu.addAction(self.tr("Append"))
         self.BP12_SSC_append_selected_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_SSC_diagram_chart, self.selections, True))
-        self.BP12_SSC_plot_all_action = self.BP12_SSC_diagram_menu.addAction(self.tr("Plot All"))
+            lambda: self.plot_chart(self.BP12_SSC_diagram, self.selections, True))
+        self.BP12_SSC_plot_all_action = self.BP12_SSC_menu.addAction(self.tr("Plot All"))
         self.BP12_SSC_plot_all_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_SSC_diagram_chart, self._dataset, False))
-        self.BP12_SSC_append_all_action = self.BP12_SSC_diagram_menu.addAction(self.tr("Append All"))
+            lambda: self.plot_chart(self.BP12_SSC_diagram, self._dataset, False))
+        self.BP12_SSC_append_all_action = self.BP12_SSC_menu.addAction(self.tr("Append All"))
         self.BP12_SSC_append_all_action.triggered.connect(
-            lambda: self.plot_chart(self.BP12_SSC_diagram_chart, self._dataset, True))
+            lambda: self.plot_chart(self.BP12_SSC_diagram, self._dataset, True))
+        self.CM_menu = self.menu.addMenu(self.tr("Plot C-M Diagram"))
+        self.CM_plot_selected_action = self.CM_menu.addAction(self.tr("Plot"))
+        self.CM_plot_selected_action.triggered.connect(
+            lambda: self.plot_chart(self.CM_diagram, self.selections, False))
+        self.CM_append_selected_action = self.CM_menu.addAction(self.tr("Append"))
+        self.CM_append_selected_action.triggered.connect(
+            lambda: self.plot_chart(self.CM_diagram, self.selections, True))
+        self.CM_plot_all_action = self.CM_menu.addAction(self.tr("Plot All"))
+        self.CM_plot_all_action.triggered.connect(
+            lambda: self.plot_chart(self.CM_diagram, self._dataset, False))
+        self.CM_append_all_action = self.CM_menu.addAction(self.tr("Append All"))
+        self.CM_append_all_action.triggered.connect(
+            lambda: self.plot_chart(self.CM_diagram, self._dataset, True))
         self.previous_button.setEnabled(False)
         self.current_page_combo_box.setEnabled(False)
         self.next_button.setEnabled(False)
         self.data_table.customContextMenuRequested.connect(self.show_menu)
-        self.frequency_curve_chart = FrequencyChart()
-        self.frequency_curve_3D_chart = Frequency3DChart()
-        self.cumulative_curve_chart = CumulativeChart()
-        self.folk54_GSM_diagram_chart = Folk54GSMDiagramChart()
-        self.folk54_SSC_diagram_chart = Folk54SSCDiagramChart()
-        self.BP12_GSM_diagram_chart = BP12GSMDiagramChart()
-        self.BP12_SSC_diagram_chart = BP12SSCDiagramChart()
+        self.cumulative_chart = CumulativeChart()
+        self.frequency_chart = FrequencyChart()
+        self.frequency_3D_chart = Frequency3DChart()
+        self.frequency_heatmap = FrequencyHeatmap()
+        self.folk54_GSM_diagram = Folk54GSMDiagramChart()
+        self.folk54_SSC_diagram = Folk54SSCDiagramChart()
+        self.BP12_GSM_diagram = BP12GSMDiagramChart()
+        self.BP12_SSC_diagram = BP12SSCDiagramChart()
+        self.CM_diagram = CMDiagramChart()
         self.normal_msg = QtWidgets.QMessageBox(self)
 
     def show_menu(self, pos):
@@ -401,38 +430,48 @@ class StatisticalAnalyzer(QtWidgets.QWidget):
             self.FW57_checkbox.setText(self.tr("Method of Statistical Moments"))
         for i, (_, description) in enumerate(self.supported_proportions):
             self.proportion_combo_box.setItemText(i, description)
-        self.plot_cumulative_curve_menu.setTitle(self.tr("Plot Cumulative Frequency Chart"))
+        self.plot_cumulative_menu.setTitle(self.tr("Plot Cumulative Frequency Chart"))
         self.cumulative_plot_selected_action.setText(self.tr("Plot"))
         self.cumulative_append_selected_action.setText(self.tr("Append"))
         self.cumulative_plot_all_action.setText(self.tr("Plot All"))
         self.cumulative_append_all_action.setText(self.tr("Append All"))
-        self.plot_frequency_curve_menu.setTitle(self.tr("Plot Frequency Distribution Chart"))
+        self.plot_frequency_menu.setTitle(self.tr("Plot Frequency Distribution Chart"))
         self.frequency_plot_selected_action.setText(self.tr("Plot"))
         self.frequency_append_selected_action.setText(self.tr("Append"))
         self.frequency_plot_all_action.setText(self.tr("Plot All"))
         self.frequency_append_all_action.setText(self.tr("Append All"))
-        self.plot_frequency_curve_3D_menu.setTitle(self.tr("Plot Frequency 3D Chart"))
+        self.plot_frequency_3D_menu.setTitle(self.tr("Plot Frequency 3D Chart"))
         self.frequency_3D_plot_selected_action.setText(self.tr("Plot"))
         self.frequency_3D_append_selected_action.setText(self.tr("Append"))
         self.frequency_3D_plot_all_action.setText(self.tr("Plot All"))
         self.frequency_3D_append_all_action.setText(self.tr("Append All"))
-        self.folk54_GSM_diagram_menu.setTitle(self.tr("Plot GSM Diagram (Folk, 1954)"))
+        self.plot_frequency_heatmap_menu.setTitle(self.tr("Plot Frequency Heatmap"))
+        self.frequency_heatmap_plot_selected_action.setText(self.tr("Plot"))
+        self.frequency_heatmap_append_selected_action.setText(self.tr("Append"))
+        self.frequency_heatmap_plot_all_action.setText(self.tr("Plot All"))
+        self.frequency_heatmap_append_all_action.setText(self.tr("Append All"))
+        self.folk54_GSM_menu.setTitle(self.tr("Plot GSM Diagram (Folk, 1954)"))
         self.folk54_GSM_plot_selected_action.setText(self.tr("Plot"))
         self.folk54_GSM_append_selected_action.setText(self.tr("Append"))
         self.folk54_GSM_plot_all_action.setText(self.tr("Plot All"))
         self.folk54_GSM_append_all_action.setText(self.tr("Append All"))
-        self.folk54_SSC_diagram_menu.setTitle(self.tr("Plot SSC Diagram (Folk, 1954)"))
+        self.folk54_SSC_menu.setTitle(self.tr("Plot SSC Diagram (Folk, 1954)"))
         self.folk54_SSC_plot_selected_action.setText(self.tr("Plot"))
         self.folk54_SSC_append_selected_action.setText(self.tr("Append"))
         self.folk54_SSC_plot_all_action.setText(self.tr("Plot All"))
         self.folk54_SSC_append_all_action.setText(self.tr("Append All"))
-        self.BP12_GSM_diagram_menu.setTitle(self.tr("Plot GSM Diagram (Blott and Pye, 2012)"))
+        self.BP12_GSM_menu.setTitle(self.tr("Plot GSM Diagram (Blott and Pye, 2012)"))
         self.BP12_GSM_plot_selected_action.setText(self.tr("Plot"))
         self.BP12_GSM_append_selected_action.setText(self.tr("Append"))
         self.BP12_GSM_plot_all_action.setText(self.tr("Plot All"))
         self.BP12_GSM_append_all_action.setText(self.tr("Append All"))
-        self.BP12_SSC_diagram_menu.setTitle(self.tr("Plot SSC Diagram (Blott and Pye, 2012)"))
+        self.BP12_SSC_menu.setTitle(self.tr("Plot SSC Diagram (Blott and Pye, 2012)"))
         self.BP12_SSC_plot_selected_action.setText(self.tr("Plot"))
         self.BP12_SSC_append_selected_action.setText(self.tr("Append"))
         self.BP12_SSC_plot_all_action.setText(self.tr("Plot All"))
         self.BP12_SSC_append_all_action.setText(self.tr("Append All"))
+        self.CM_menu.setTitle(self.tr("Plot C-M Diagram"))
+        self.CM_plot_selected_action.setText(self.tr("Plot"))
+        self.CM_append_selected_action.setText(self.tr("Append"))
+        self.CM_plot_all_action.setText(self.tr("Plot All"))
+        self.CM_append_all_action.setText(self.tr("Append All"))
